@@ -1,4 +1,4 @@
-// $Id: Flu_File_Chooser.h,v 1.60 2004/07/06 18:46:31 jbryan Exp $
+// $Id: Flu_File_Chooser.h,v 1.63 2004/10/18 15:14:58 jbryan Exp $
 
 /***************************************************************
  *                FLU - FLTK Utility Widgets 
@@ -31,16 +31,17 @@
 #include "FLU/Flu_Combo_List.h"
 #include "FLU/flu_export.h"
 #include "FLU/FluSimpleString.h"
-#include "FLU/VectorClass.h"
+#include "FLU/FluVectorClass.h"
+
+FluMakeVectorClass( FluSimpleString, FluStringVector );
 
 FLU_EXPORT const char* flu_file_chooser( const char *message, const char *pattern, const char *filename );
+FLU_EXPORT int flu_multi_file_chooser( const char *message, const char *pattern, const char *filename, FluStringVector *filelist );
 FLU_EXPORT const char* flu_save_chooser( const char *message, const char *pattern, const char *filename );
 FLU_EXPORT const char* flu_dir_chooser( const char *message, const char *filename );
 FLU_EXPORT const char* flu_dir_chooser( const char *message, const char *filename, bool showFiles );
 FLU_EXPORT const char* flu_file_and_dir_chooser( const char *message, const char *filename );
 
-MakeVectorClass( FluSimpleString, StringVector );
-
 //! A file and directory choosing widget that looks and acts similar to the stock Windows file chooser
 class FLU_EXPORT Flu_File_Chooser : public Fl_Double_Window
 {
@@ -126,12 +127,12 @@ class FLU_EXPORT Flu_File_Chooser : public Fl_Double_Window
 
   //! Chooser type
   enum { 
-    SINGLE = 0,    /*!< Choose a single file or directory */
-    MULTI = 1,     /*!< Choose multiple files or directories */
-    DIRECTORY = 4, /*!< Choose directories (choosing files is implicit if this bit is clear) */
-    DEACTIVATE_FILES = 8, /*!< When choosing directories, also show the files in a deactivated state */
-    SAVING = 16,   /*!< When choosing files, whether to keep the current filename always in the input area */
-    STDFILE = 32
+    SINGLE = 0,            /*!< Choose a single file or directory */
+    MULTI = 1,             /*!< Choose multiple files or directories */
+    DIRECTORY = 4,         /*!< Choose directories (choosing files is implicit if this bit is clear) */
+    DEACTIVATE_FILES = 8,  /*!< When choosing directories, also show the files in a deactivated state */
+    SAVING = 16,           /*!< When choosing files, whether to keep the current filename always in the input area */
+    STDFILE = 32           /*!< Choose both files and directories at the same time */
   };
 
   //! Structure holding the info needed for custom file types
@@ -293,11 +294,11 @@ class FLU_EXPORT Flu_File_Chooser : public Fl_Double_Window
       inline ContextHandler& operator =( const ContextHandler &c )
       { ext = c.ext; name = c.name; type = c.type; callback = c.callback; callbackData = c.callbackData; return *this; }
     };
-  MakeVectorClass( ContextHandler, ContextHandlerVector );
+  FluMakeVectorClass( ContextHandler, ContextHandlerVector );
   static ContextHandlerVector contextHandlers;
 
   typedef PreviewWidgetBase* pPreviewWidgetBase;
-  MakeVectorClass( pPreviewWidgetBase, PreviewHandlerVector );
+  FluMakeVectorClass( pPreviewWidgetBase, PreviewHandlerVector );
   static PreviewHandlerVector previewHandlers;
 
   Fl_Check_Button *hiddenFiles;
@@ -338,7 +339,7 @@ class FLU_EXPORT Flu_File_Chooser : public Fl_Double_Window
     { ((Flu_File_Chooser*)arg)->cd( ((Flu_File_Chooser*)arg)->delayedCd.c_str() ); }
 
   inline static void selectCB( void *arg )
-    { ((Flu_File_Chooser*)arg)->hide(); }
+    { ((Flu_File_Chooser*)arg)->okCB(); }
 
   inline static void _cancelCB( Fl_Widget*, void *arg )
     { ((Flu_File_Chooser*)arg)->cancelCB(); }
@@ -544,9 +545,9 @@ class FLU_EXPORT Flu_File_Chooser : public Fl_Double_Window
 
   FluSimpleString formatDate( const char *d );
 
-  void recursiveScan( const char *dir, StringVector *files );
+  void recursiveScan( const char *dir, FluStringVector *files );
 
-  bool stripPatterns( FluSimpleString s, StringVector* patterns );
+  bool stripPatterns( FluSimpleString s, FluStringVector* patterns );
 
   int popupContextMenu( Entry *entry );
 
@@ -581,7 +582,7 @@ class FLU_EXPORT Flu_File_Chooser : public Fl_Double_Window
   bool filenameEnterCallback, filenameTabCallback, walkingHistory, caseSort, fileEditing;
   int sortMethod;
 
-  StringVector patterns;
+  FluStringVector patterns;
 
   static FileTypeInfo *types;
   static int numTypes;
