@@ -1,4 +1,4 @@
-// $Id: Flu_Tree_Browser.h,v 1.87 2004/08/02 14:18:16 jbryan Exp $
+// $Id: Flu_Tree_Browser.h,v 1.91 2004/11/05 17:03:20 jbryan Exp $
 
 /***************************************************************
  *                FLU - FLTK Utility Widgets 
@@ -20,7 +20,7 @@
 #include <string.h>
 #include <stdlib.h>
 
-#define USE_FLU_DND
+//#define USE_FLU_DND
 
 /* fltk includes */
 #include <FL/Fl.H>
@@ -66,9 +66,7 @@ class FLU_EXPORT Flu_Tree_Browser : public Fl_Group
     \return a pointer to the Node of the added entry or NULL if the add failed */
   Node* add( const char* fullpath, Fl_Widget *w = 0, bool showLabel = true );
 
-  //! Add entry \b name in path \b path to the tree. If \b w is not \c NULL then that widget is the entry and its label is visible depending on the value of \b showLabel. Note that the widget is destroyed by the tree/node on clear() or the destructor
-  /*! If \b name ends in a slash ("/"), then the entry is added as a branch, else it is added as a leaf
-    \return a pointer to the Node of the added entry or NULL if the add failed */
+  //! Convenience function that is the same as add() except \b path and \b name are concatenated (with a '/' added if necessary) to create the full path
   Node* add( const char* path, const char* name, Fl_Widget *w = 0, bool showLabel = true );
 
   //! Add entry \b name to node \b n. If \b w is not \c NULL then that widget is the entry and its label is visible depending on the value of \b showLabel. Note that the widget is destroyed by the tree/node on clear() or the destructor
@@ -80,7 +78,7 @@ class FLU_EXPORT Flu_Tree_Browser : public Fl_Group
   //! Convenience function that is the same as add() except it appends a '/' to \b fullpath if one does not exist
   Node* add_branch( const char* fullpath, Fl_Widget *w = 0, bool showLabel = true );
 
-  //! Convenience function that is the same as add() except it appends a '/' to \b name if one does not exist
+  //! Convenience function that is the same as add() except \b path and \b name are concatenated (with a '/' added if necessary) to create the full path
   Node* add_branch( const char* path, const char* name, Fl_Widget *w = 0, bool showLabel = true );
 
   //! Convenience function that is the same as add() except it appends a '/' to \b name if one does not exist
@@ -90,7 +88,7 @@ class FLU_EXPORT Flu_Tree_Browser : public Fl_Group
   //! Convenience function that is the same as add() except it removes any '/' at the end of \b fullpath
   Node* add_leaf( const char* fullpath, Fl_Widget *w = 0, bool showLabel = true );
 
-  //! Convenience function that is the same as add() except it removes any '/' at the end of \b name
+  //! Convenience function that is the same as add() except \b path and \b name are concatenated (with a '/' added if necessary) to create the full path
   Node* add_leaf( const char* path, const char* name, Fl_Widget *w = 0, bool showLabel = true );
 
   //! Convenience function that is the same as add() except it removes any '/' at the end of \b fullpath
@@ -174,6 +172,7 @@ class FLU_EXPORT Flu_Tree_Browser : public Fl_Group
     { rdata.defBranchColor = color; rdata.defBranchFont = font; rdata.defBranchSize = size; }
 
   //! Set the default branch icons to use for all subsequent branches added to the tree
+  /*! Passing in \c NULL for either icon will disable that icon for the branch */
   void branch_icons( Fl_Image *closed, Fl_Image *open );
 
   //! Get the type of box to draw the browser in
@@ -317,6 +316,10 @@ class FLU_EXPORT Flu_Tree_Browser : public Fl_Group
   inline void frame_rate( float f )
     { if( f <= 0.0f ) rdata.fps = 0.001f; else rdata.fps = f; }
 
+  //! \return the currently highlighted node
+  inline Node* get_hilighted()
+    { return rdata.hilighted; }
+
   //! \return a pointer to the root node of the tree
   inline Node *get_root() { return &root; }
 
@@ -503,6 +506,14 @@ class FLU_EXPORT Flu_Tree_Browser : public Fl_Group
   inline int selection_drag_mode() const
     { return rdata.selectionDragMode; }
 
+  //! Set whether the hilighted node is always selected. Default is \c false
+  inline void selection_follows_hilight( bool b )
+    { rdata.selectionFollowsHilight = b; if( b && rdata.hilighted ) rdata.hilighted->select(true); }
+
+  //! Get whether the hilighted node is always selected
+  inline bool selection_follows_hilight()
+    { return rdata.selectionFollowsHilight; }
+
   //! Set how individual entries are selected using the mouse. This can be one of FLU_NO_SELECT, FLU_SINGLE_SELECT, FLU_MULTI_SELECT. Default is FLU_MULTI_SELECT
   inline void selection_mode( int m )
     { rdata.selectionMode = m; root.unselect_all(); }
@@ -511,12 +522,17 @@ class FLU_EXPORT Flu_Tree_Browser : public Fl_Group
   inline int selection_mode() const
     { return rdata.selectionMode; }
 
+  //! If selection_mode() is \c FLU_SINGLE_SELECT, then whichever node is under the mouse will always be selected, with all others unselected. Default is \c false
   inline void select_under_mouse( bool b )
     { rdata.selectUnderMouse = b; }
 
+  //! If selection_mode() is \c FLU_SINGLE_SELECT, then whichever node is under the mouse will always be selected, with all others unselected. Default is \c false
   inline bool select_under_mouse() const
     { return rdata.selectUnderMouse; }
 
+  //! Calling this will cause any newly added branches to use the default branch icon
+  void set_default_branch_icons();
+
   //! Set which node is hilighted and ready to be selected or unselected. This also scrolls the browser so \b n is visible.
   void set_hilighted( Node* n );
 
@@ -659,8 +675,8 @@ class FLU_EXPORT Flu_Tree_Browser : public Fl_Group
   public:
     // volatile objects (from the perspective of each node during a recursive descent)
     int x, y, totalW, totalH;
-    bool first, last, dragging, shiftSelect, shiftSelectAll, visibilityChanged;
-    Node *hilighted, *lastHilighted, *previous, *grabbed, *dragNode, *animatedNode;
+    bool first, last, dragging, shiftSelect, shiftSelectAll, visibilityChanged, selectionFollowsHilight;
+    Node *hilighted, /**lastHilighted, */ *previous, *grabbed, *dragNode, *animatedNode;
     int delta, shadedIndex, counter, searchIndex, branchIconW, dragPos, dragWhere;
     Fl_Color lineColor, bgColor, selectionColor;
     bool forceResize;  // force the browser to resize on the next draw (which forces a recalculation of the tree layout)
@@ -725,7 +741,9 @@ class FLU_EXPORT Flu_Tree_Browser : public Fl_Group
       // flags
       enum { SELECTED = 0x0001, COLLAPSED = 0x0002, LEAF = 0x0004, SHOW_LABEL = 0x0008,
 	     ACTIVE = 0x0010, EXPAND_TO_WIDTH = 0x0020, ALWAYS_OPEN = 0x0040,
-	     SOME_VISIBLE_CHILDREN = 0x0080, MOVABLE = 0x0100, DROPPABLE = 0x0200 };
+	     SOME_VISIBLE_CHILDREN = 0x0080, MOVABLE = 0x0100, DROPPABLE = 0x0200,
+	     AUTO_LABEL_COLOR = 0x0400, AUTO_COLOR = 0x0800, AUTO_LABEL = 0x1000, 
+	     SWAP_LABEL_AND_WIDGET = 0x2000, ICON_AT_END = 0x4000 };
 
       // flag manipulator functions
       inline bool CHECK( unsigned short flag ) const { return flags & flag; }
@@ -757,6 +775,15 @@ class FLU_EXPORT Flu_Tree_Browser : public Fl_Group
       //! Convenience function that is the same as add() except it removes any '/' at the end of \b fullpath
       Node* add_leaf( const char* fullpath, Fl_Widget *w = 0, bool showLabel = true );
 
+      //! Convenience function that is the same as add() except \b path and \b name are concatenated (with a '/' added if necessary) to create the full path
+      Node* add( const char* path, const char* name, Fl_Widget *w = 0, bool showLabel = true );
+
+      //! Convenience function that is the same as add_branch() except \b path and \b name are concatenated (with a '/' added if necessary) to create the full path
+      Node* add_branch( const char* path, const char* name, Fl_Widget *w = 0, bool showLabel = true );
+
+      //! Convenience function that is the same as add_leaf() except \b path and \b name are concatenated (with a '/' added if necessary) to create the full path
+      Node* add_leaf( const char* path, const char* name, Fl_Widget *w = 0, bool showLabel = true );
+
       //! Set whether a branch node is always open (only for branch nodes). Default is \c false
       inline void always_open( bool b )
 	{ if( b ) open(true); SET(ALWAYS_OPEN,b); tree->rdata.forceResize = true; }
@@ -765,6 +792,30 @@ class FLU_EXPORT Flu_Tree_Browser : public Fl_Group
       inline bool always_open() const
 	{ return CHECK(ALWAYS_OPEN); }
 
+      //! Set whether the color of the widget in this node tracks the color of the node itself. This is useful for changing the color of the widget when the node is selected. Default is \c false
+      inline void auto_color( bool b )
+	{ SET(AUTO_COLOR,b); }
+
+      //! Get whether the color of the widget in this node tracks the color of the node itself
+      inline bool auto_color()
+	{ return CHECK(AUTO_COLOR); }
+
+      //! Set whether the label of the widget in this node tracks the label of the node itself. This is useful for when the label of the node changes and you want the widget to reflect the change
+      inline void auto_label( bool b )
+	{ SET(AUTO_LABEL,b); }
+
+      //! Set whether the label of the widget in this node tracks the label of the node itself.
+      inline bool auto_label()
+	{ return CHECK(AUTO_LABEL); }
+
+      //! Set whether the label color of the widget in this node tracks the label color of the node itself. This is useful for changing the label color of the widget when the node is selected. Default is \c false
+      inline void auto_label_color( bool b )
+	{ SET(AUTO_LABEL_COLOR,b); }
+
+      //! Get whether the label color of the widget in this node tracks the label color of the node itself
+      inline bool auto_label_color()
+	{ return CHECK(AUTO_LABEL_COLOR); }
+
       //! Set the branch icons to use for this node (only for branch nodes)
       void branch_icons( Fl_Image *closed, Fl_Image *open );
 
@@ -812,11 +863,11 @@ class FLU_EXPORT Flu_Tree_Browser : public Fl_Group
       inline bool droppable()
 	{ return CHECK(DROPPABLE); }
 
-      //! Set whether to force the size of the widget to expand to fill the visible width of the browser. Default is \c false
+      //! Set whether to force the size of the embedded widget to expand to fill the visible width of the browser. Default is \c false
       inline void expand_to_width( bool b )
 	{ SET(EXPAND_TO_WIDTH,b); tree->rdata.forceResize = true; }
 
-      //! Get whether to force the size of the widget to expand to fill the visible width of the browser
+      //! Get whether to force the size of the embedded widget to expand to fill the visible width of the browser
       inline bool expand_to_width() const
 	{ return CHECK(EXPAND_TO_WIDTH); }
 
@@ -852,10 +903,26 @@ class FLU_EXPORT Flu_Tree_Browser : public Fl_Group
       //! \return the first leaf encountered in a depth-first traversal (or this node if it is a leaf). NULL means there are no leaves
       Node* first_leaf();
 
+      //! Set whether this node draws the label to the left of the widget (\c false, default) or to the right of the widget (\c true)
+      inline void swap_label_and_widget( bool b )
+	{ SET(SWAP_LABEL_AND_WIDGET,b); }
+
+      //! Get whether this node draws the label to the left of the widget (\c false, default) or to the right of the widget (\c true)
+      inline bool swap_label_and_widget()
+	{ return CHECK(SWAP_LABEL_AND_WIDGET); }      
+
       //! \return the selected Node that is at \b index among all selected nodes, or \c NULL if no Node is selected
       /*! For example, \c get_selected(1) will return the first selected node. */
       Node* get_selected( int index );
 
+      //! Set whether the icon for this node is drawn after the label and widget (\c true) or before (\c false, default) (only for leaf nodes)
+      inline void icon_at_end( bool b )
+	{ SET(ICON_AT_END,b); }
+
+      //! Get whether the icon for this node is drawn after the label and widget (\c true) or before (\c false, default) (only for leaf nodes)
+      inline bool icon_at_end()
+	{ return CHECK(ICON_AT_END); }
+
       //! Get the unique ID of this node
       inline unsigned int id() const
 	{ return _id; }
@@ -1017,7 +1084,7 @@ class FLU_EXPORT Flu_Tree_Browser : public Fl_Group
       /*! \return the unique id of the removed entry, or \c 0 if no matching entry was found */
       unsigned int remove( unsigned int id );
 
-      //! Remove the entry containing the widget \b w from this node. Note that the widget is automatically destroyed
+      //! Remove the node containing the widget \b w from this node. Note that the widget is automatically destroyed
       /*! \return the unique id of the removed entry, or \c 0 if no matching entry was found */
       unsigned int remove( Fl_Widget *w );
 
@@ -1047,7 +1114,7 @@ class FLU_EXPORT Flu_Tree_Browser : public Fl_Group
       //! Swap this node and node \b n in their respective trees
       /*! \return \c true if the swap was successful, else \c false */
       inline bool swap( Node* n )
-	{ swap( this, n ); }
+	{ return swap( this, n ); }
 
       //! Swap nodes \b n1 and \b n2 in their respective trees
       /*! \return \c true if the swap was successful, else \c false */
@@ -1167,7 +1234,7 @@ class FLU_EXPORT Flu_Tree_Browser : public Fl_Group
   Fl_Group *_box;
   Node root;
   RData rdata;
-  int lastEvent;
+  //int lastEvent;
   float autoScrollX, autoScrollY;
   bool scrolledTimerOn;
 
