@@ -1,4 +1,4 @@
-// $Id: Flu_Label.cpp,v 1.4 2004/03/28 14:06:57 jbryan Exp $
+// $Id: Flu_Label.cpp,v 1.6 2004/10/21 15:21:07 jbryan Exp $
 
 /***************************************************************
  *                FLU - FLTK Utility Widgets 
@@ -20,10 +20,11 @@ Flu_Label :: Flu_Label( int x, int y, int w, int h, const char* l )
   : Fl_Box( x, y, w, h, 0 )
 {
   _autoSize = false;
-  align( FL_ALIGN_LEFT | FL_ALIGN_INSIDE );
+  align( FL_ALIGN_LEFT | FL_ALIGN_WRAP | FL_ALIGN_INSIDE );
   _label = NULL;
   label( l );
   box( FL_NO_BOX );
+  clear_visible_focus();
 }
 
 Flu_Label :: ~Flu_Label()
