@@ -1,4 +1,4 @@
-// $Id: Flu_Tree_Browser.cpp,v 1.121 2004/08/02 14:18:16 jbryan Exp $
+// $Id: Flu_Tree_Browser.cpp,v 1.126 2004/11/05 17:03:20 jbryan Exp $
 
 /***************************************************************
  *                FLU - FLTK Utility Widgets 
@@ -654,7 +654,7 @@ Flu_Tree_Browser :: Flu_Tree_Browser( int x, int y, int w, int h, const char *l
   , Flu_DND( "Flu_Tree_Browser" )
 #endif
 {
-  lastEvent = -1;
+  //lastEvent = -1;
   autoScrollX = autoScrollY = 0.0f;
 #ifdef USE_FLU_DND
   dnd_allow_type( "Flu_Tree_Browser" );
@@ -666,6 +666,7 @@ Flu_Tree_Browser :: Flu_Tree_Browser( int x, int y, int w, int h, const char *l
   _box = new Fl_Group( x, y, w-SCROLL_SIZE, h-SCROLL_SIZE );
   _box->resizable( NULL );
   _box->end();
+  //_box->set_output();
   scrollV = new Fl_Scrollbar( x+w-SCROLL_SIZE, y, SCROLL_SIZE, h-SCROLL_SIZE );
   scrollV->type( FL_VERTICAL );
   scrollV->callback( _scrollCB, this );
@@ -700,6 +701,7 @@ Flu_Tree_Browser :: Flu_Tree_Browser( int x, int y, int w, int h, const char *l
   end();
 
   // set the default values for the tree
+  selection_follows_hilight( false );
   select_under_mouse( false );
   open_without_children( true );
   auto_branches( false );
@@ -772,17 +774,23 @@ void Flu_Tree_Browser :: collapse_icons( Fl_Image *closed, Fl_Image *open )
 
 void Flu_Tree_Browser :: branch_icons( Fl_Image *closed, Fl_Image *open )
 {
-  if( closed )
-    rdata.branchIcons[0] = closed;
+  //if( closed )
+  rdata.branchIcons[0] = closed;
   //else
   //rdata.branchIcons[0] = rdata.defaultBranchIcons[0];
 
-  if( open )
-    rdata.branchIcons[1] = open;
+  //if( open )
+  rdata.branchIcons[1] = open;
   //else
   //rdata.branchIcons[1] = rdata.defaultBranchIcons[1];
 }
 
+void Flu_Tree_Browser :: set_default_branch_icons()
+{
+  rdata.branchIcons[0] = rdata.defaultBranchIcons[0];
+  rdata.branchIcons[1] = rdata.defaultBranchIcons[1];
+}
+
 void Flu_Tree_Browser :: leaf_icon( Fl_Image *icon )
 {
   rdata.leafIcon = icon;
@@ -1093,23 +1101,23 @@ int Flu_Tree_Browser :: handle( int event )
   if( event == FL_NO_EVENT )//|| event == FL_MOVE )
     return 0;
 
-  if( event == FL_FOCUS && rdata.lastHilighted )
+  if( event == FL_FOCUS )//&& rdata.lastHilighted )
     {
-      set_hilighted( rdata.lastHilighted );
-      lastEvent = event;
-      Fl_Group::handle( event );
+      //set_hilighted( rdata.lastHilighted );
+      //lastEvent = event;
+      //Fl_Group::handle( event );
       redraw();
       return 1;
     }
 
   if( event == FL_UNFOCUS )
     {
-      if( lastEvent != FL_LEAVE )
-	{
-	  rdata.lastHilighted = rdata.hilighted;
-	}
-      set_hilighted( NULL );
-      lastEvent = event;
+      //if( lastEvent != FL_LEAVE )
+      //{
+	  //rdata.lastHilighted = rdata.hilighted;
+      //}
+      //set_hilighted( NULL );
+      //lastEvent = event;
       Fl_Group::handle( event );
       redraw();
       return 1;
@@ -1119,21 +1127,24 @@ int Flu_Tree_Browser :: handle( int event )
     {
       if( ! (event == FL_MOVE || event == FL_ENTER || event == FL_LEAVE ) )
 	_box->redraw();
+
       if( Fl_Group::handle( event ) )
 	{
 	  //if( event == FL_KEYDOWN || event == FL_KEYUP )
-	    // redraw();
+	  // redraw();
 	  return 1;
 	}
+      //if (scrollV && Fl::event_inside(scrollV) && scrollV->handle(event)) return 1;
+      //if (scrollH && Fl::event_inside(scrollH) && scrollH->handle(event)) return 1;
     }
 
   if( event == FL_RELEASE )
     {
-      Fl::focus(this);
+      //Fl::focus(this);
       rdata.dragging = false;
       rdata.grabbed = 0;
       rdata.dragNode = 0;
-      redraw();
+      //redraw();
     }
 
   int dx = Fl::box_dx(box()), dy = Fl::box_dy(box());
@@ -1211,7 +1222,7 @@ int Flu_Tree_Browser :: handle( int event )
   int val = root.recurse( rdata, Node::HANDLE, event );
   if( val )
     {
-      redraw();
+      //redraw();
       if( rdata.visibilityChanged )
 	root.determineVisibility();
       if( val == 1 )
@@ -1254,6 +1265,13 @@ void Flu_Tree_Browser :: set_hilighted( Flu_Tree_Browser::Node* n )
 
   if( rdata.hilighted )
     {
+      if( rdata.selectionFollowsHilight )
+	{
+	  if( rdata.selectionMode == FLU_SINGLE_SELECT )
+	    unselect_all();
+	  rdata.hilighted->select( true );
+	}
+
       int extraH = scrollH->visible() ? scrollH->h() : 0;
 
       // if the hilighted entry is below the visible bounds of the browser, move the vertical scrollbar
@@ -1376,7 +1394,7 @@ Flu_Tree_Browser :: Node :: ~Node()
   if( tree )
     {
       if( tree->rdata.hilighted == this ) tree->rdata.hilighted = NULL;
-      if( tree->rdata.lastHilighted == this ) tree->rdata.lastHilighted = NULL;
+      //if( tree->rdata.lastHilighted == this ) tree->rdata.lastHilighted = NULL;
       if( tree->rdata.grabbed == this ) tree->rdata.grabbed = NULL;
       if( tree->rdata.dragNode == this ) tree->rdata.dragNode = NULL;
     }
@@ -1865,7 +1883,7 @@ void Flu_Tree_Browser :: Node :: draw( RData &rdata, bool measure )
 	draw_Ldash( X-halfHGap, Y-halfVGap, lIcon->w()+rdata.hGap, currentH+rdata.vGap );
 
       // draw the leaf icon
-      if( lIcon )
+      if( lIcon && !CHECK(ICON_AT_END) )
 	{
 	  if( doDraw )
 	    lIcon->draw( X, Y+(currentH>>1)-(lIcon->h()>>1) );
@@ -1952,13 +1970,15 @@ void Flu_Tree_Browser :: Node :: draw( RData &rdata, bool measure )
 	    bIcon[which]->draw( X, Y+(currentH>>1)-(bIcon[which]->h()>>1) );
 	  X += bIcon[which]->w() + rdata.wGap;
 	}
+      else
+	X += rdata.wGap;
     }
 
   if( doDraw )
     fl_line_style( 0 );
 
   // draw the entry
-  if( CHECK(SHOW_LABEL) )
+  if( CHECK(SHOW_LABEL) && !CHECK(SWAP_LABEL_AND_WIDGET) )
     {
       if( doDraw )
 	{
@@ -1969,12 +1989,19 @@ void Flu_Tree_Browser :: Node :: draw( RData &rdata, bool measure )
 	}
       X += textW;
     }
+
   if( _widget )
     {
       int widgetW = _widget->w->w();
       int widgetH = _widget->w->h();
       if( doDraw )
 	{
+	  if( CHECK(AUTO_COLOR) )
+	    _widget->w->color( bgColor );
+	  if( CHECK(AUTO_LABEL_COLOR) )
+	    _widget->w->labelcolor( tColor );
+	  if( CHECK(AUTO_LABEL) )
+	    _widget->w->label( text.c_str() );
 	  _widget->w->redraw();
 	  _widget->w->position( X, Y+(currentH>>1)-(widgetH>>1) );
 	  if( CHECK(EXPAND_TO_WIDTH) )
@@ -1990,8 +2017,28 @@ void Flu_Tree_Browser :: Node :: draw( RData &rdata, bool measure )
 	X += widgetW;
     }
 
+  if( CHECK(SHOW_LABEL) && CHECK(SWAP_LABEL_AND_WIDGET) )
+    {
+      if( doDraw )
+	{
+	  fl_draw_box( FL_FLAT_BOX, X, Y+(currentH>>1)-(textH>>1), textW, textH, bgColor );
+	  fl_color( tColor );
+	  fl_font( textFont, textSize );
+	  fl_draw( text.c_str(), X, Y+(currentH>>1)-(textH>>1), textW, textH, FL_ALIGN_LEFT );
+	}
+      X += textW;
+    }
+
+  // draw the leaf icon to the right of the label and widget
+  if( is_leaf() && lIcon && CHECK(ICON_AT_END) )
+    {
+      if( doDraw )
+	lIcon->draw( X, Y+(currentH>>1)-(lIcon->h()>>1) );
+      X += lIcon->w() + rdata.wGap;
+    }
+
   // if hilighted, draw a box outlining the entry
-  if( rdata.hilighted == this && doDraw )
+  if( Fl::focus() == tree && rdata.hilighted == this && doDraw )
     {
       fl_color( hilightColor );
       fl_line_style( FL_DOT, 1 );
@@ -2549,6 +2596,9 @@ int Flu_Tree_Browser :: Node :: recurse( RData &rdata, int type, int event )
 	  }
 #endif
 
+	//if( _widget && _widget->w && Fl::event_inside(_widget->w) && _widget->w->handle(event))
+	//return 1;
+
 	// single selection
 	if( rdata.selectionMode == FLU_SINGLE_SELECT )
 	  {
@@ -2600,10 +2650,13 @@ int Flu_Tree_Browser :: Node :: recurse( RData &rdata, int type, int event )
 			open( true );
 		      }
 		  }
+		Fl::focus(tree);
 		return 1;
 	      }
 	    else if( event == FL_DRAG )
 	      {
+		if( rdata.selectionDragMode == FLU_DRAG_IGNORE )
+		  return 1;
 		rdata.dragging = true;
 		//if( ( rdata.selectionDragMode == FLU_DRAG_IGNORE || rdata.selectionDragMode == FLU_DRAG_TO_MOVE) && ( tree->insertion_mode() == FLU_INSERT_FRONT || tree->insertion_mode() == FLU_INSERT_BACK ) )
 		//return 1;
@@ -2698,10 +2751,13 @@ int Flu_Tree_Browser :: Node :: recurse( RData &rdata, int type, int event )
 		      }
 		    tree->set_hilighted( this );
 		  }
+		Fl::focus(tree);
 		return 1;
 	      }
 	    else if( event == FL_DRAG )
 	      {
+		if( rdata.selectionDragMode == FLU_DRAG_IGNORE )
+		  return 1;
 		rdata.dragging = true;
 		//if( ( rdata.selectionDragMode == FLU_DRAG_IGNORE || rdata.selectionDragMode == FLU_DRAG_TO_MOVE) && ( tree->insertion_mode() == FLU_INSERT_FRONT || tree->insertion_mode() == FLU_INSERT_BACK ) )
 		//return 1;
@@ -2962,6 +3018,7 @@ unsigned int Flu_Tree_Browser :: Node :: remove( unsigned int id )
 	  delete n;
 	  if( tree->rdata.autoBranches )
 	    initType();
+	  tree->redraw();
 	  return id;
 	}
       else if( n->remove( id ) )
@@ -2995,6 +3052,7 @@ unsigned int Flu_Tree_Browser :: Node :: remove( Fl_Widget *w )
 	      delete n;
 	      if( tree->rdata.autoBranches )
 		initType();
+	      tree->redraw();
 	      return id;
 	    }
 	}
@@ -3253,6 +3311,30 @@ Flu_Tree_Browser::Node* Flu_Tree_Browser :: Node :: add_leaf( const char* fullpa
   return add( p.c_str(), w, showLabel );
 }
 
+Flu_Tree_Browser::Node* Flu_Tree_Browser :: Node :: add( const char* path, const char* name, Fl_Widget *w, bool showLabel )
+{
+  FluSimpleString p( path );
+  if( p.size() && p[p.size()-1] != '/' && p[p.size()-1] != '\\' ) p += "/";
+  p += name;
+  return add( p.c_str(), w, showLabel );
+}
+
+Flu_Tree_Browser::Node* Flu_Tree_Browser :: Node :: add_branch( const char* path, const char* name, Fl_Widget *w, bool showLabel )
+{
+  FluSimpleString p( path );
+  if( p.size() && p[p.size()-1] != '/' && p[p.size()-1] != '\\' ) p += "/";
+  p += name;
+  return add_branch( p.c_str(), w, showLabel );
+}
+
+Flu_Tree_Browser::Node* Flu_Tree_Browser :: Node :: add_leaf( const char* path, const char* name, Fl_Widget *w, bool showLabel )
+{
+  FluSimpleString p( path );
+  if( p.size() && p[p.size()-1] != '/' && p[p.size()-1] != '\\' ) p += "/";
+  p += name;
+  return add_leaf( p.c_str(), w, showLabel );
+}
+
 Flu_Tree_Browser::Node* Flu_Tree_Browser :: Node :: modify( const char* path, int what, RData &rdata, Fl_Widget *w, bool showLabel )
 {
   // find the selected entry at rdata.searchIndex among all selected entries
@@ -3425,6 +3507,7 @@ Flu_Tree_Browser::Node* Flu_Tree_Browser :: Node :: modify( const char* path, in
 
 	    if( tree->rdata.autoBranches )
 	      initType();
+	    tree->redraw();
 	  }
 	// otherwise recurse on the remainder of the path
 	else
